<?php

class Category	{
	var $name; /** カテゴリの名前です。 **/
	var $dir; /** 使用するディレクトリ名です。 **/
	var $engine; /** 使用するカテゴリです。 **/
	var $img; /** 画像設定の連想配列です。 **/
	var $capture; /** 画像撮影設定の連想配列です。 **/
	var $picture; /** 画像表示設定の連想配列です。 **/
	var $isDefault; /** デフォルトカテゴリの条件に合致すれば true **/
	var $_depth = 1; /** 階層の深さレベル **/
	var $_children = array(); /** 子カテゴリの配列 **/
	var $_parent = null; /** 親カテゴリ **/
	
	function Category($name, $dir, $engine, $img, $cap, $pic, $isDefault ){
		$this->name = $name;
		$this->dir = $dir;
		$this->engine = $engine;
		$this->img = $img;
		$this->capture = $cap;
		$this->picture = $pic;
		$this->isDefault = $isDefault;
	}
	
	/**
	* @return int
	*
	* カテゴリの階層の深さを返します。
	**/
	function getDepth(){
		return $this->_depth;
	}
	
	/**
	* @param int
	*
	* カテゴリの階層の深さを設定します。
	**/
	function setDepth( $depth ){
		return $this->_depth = $depth;
	}
	
	/**
	* @return Category
	*
	* このカテゴリが属する最上位のカテゴリを求めて返す。
	**/
	function getOwnerCategory(){
		if( $this->_parent ===  null ){
			return $this;
		} else {
			return $this->_parent->getOwnerCategory();
		}
	}
	
	/**
	* @param bool 画像全体のディレクトリ部分を含むなら true 含まないなら false(デフォルト)
	* @return string
	* 
	* カテゴリが使用する画像のディレクトリパスを返す。
	* 画像全体のディレクトリ部分まで含めるなら @param を true に。
	**/
	function getPath( $containsImage = false ){
		if( $this->_parent === null ){
			if( $containsImage ){
				return ($this->img["dir"]."/".$this->dir);
			} else {
				return $this->dir;
			}
		} else {
			return ($this->_parent->getPath( $containsImage )."/".$this->dir);
		}
	}
	
	/**
	* @return array 直系の子カテゴリの配列です。
	*
	**/
	function getChildren(){
		return $this->_children;
	}
	
	/*
	* @param 子となるカテゴリオブジェクト
	*
	* 子を追加します。
	* ここで $category->setParent($this); すれば親は自ずと設定されるはずだったのだが
	* 何故かうまくいかなかった。
	*/
	function addChild( $category ){
		array_push( $this->_children, $category );
	}
	
	/**
	* @param 親カテゴリオブジェクト
	*
	* 親カテゴリを設定します。
	* 階層の深さはこのメソッドだけが変更します。
	**/
	function setParent( &$pc ){
		$this->_parent = $pc;
		$this->_depth = $pc->getDepth()+1;
	}
	
	function toHTML( $unlinkPath = null ){
		$tab = str_repeat( "\t", $this->getDepth() );
		$html = $tab;
		$path = $this->getPath();
		if( $path === $unlinkPath or empty($this->_children) == false ){
			$html .= "<li>".$this->name."</li>\n";
		} else {
			$html .= "<li><a href='".basename($_SERVER['PHP_SELF'])."?c=".$path."'>".$this->name."</a></li>\n";
		}
		foreach( $this->_children as $child ){
			$html .= $tab."<li><ul>\n".$child->toHTML( $unlinkPath ).$tab."</ul></li>\n";
		}
		return $html;
	}
}

?>
