<?php

/*
"THUMBLINK" Type C : サムネイル付きリンク集 XML 設定版
*/

error_reporting(E_ALL);
mb_internal_encoding("UTF-8");

require_once("./lib/BasicFunction.php");
require_once("./lib/HttpFunction.php");
require_once("./lib/HttpResponse.php");
require_once("./lib/SerializedFile.php");
require_once("./lib/PageIterator.class.php");
require_once("./Page.class.php");
require_once("./Category.class.php");

define("THUMBLINK_INFO", "Thumblink type C v0.5.0");

class Thumblink extends HttpResponse{
	var $option;
	var $pages;
	var $categories;
	var $categorizedPage;
	
	/*
	* @access : public
	* @return : void
	* @description : 
	* 主処理の実行。
	*/
	function doGet(){
		$this->header();
		$this->prepare();
		$this->sanitize();
		$this->output();
	}
	
	function doHead(){
		$this->header();
	}
	
	/*
	* @access : protected
	* @return : void
	* @description : 
	* HTTP ヘッダーの出力。HEAD リクエストならここで処理終了
	*/
	function header(){
		if( empty( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) == false ){
			if( (abs(filemtime( "./pages" ) - @strtotime( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ))) <= TIME_MINUTE * 3 ){
				header("HTTP/1.1 304 Not Modified");
				exit();
			}
		}
		if( isAcceptable("application/xhtml+xml") == true ){
			header("Content-Type: application/xhtml+xml; charset=utf-8");
		} else {
			header("Content-Type: text/html; charset=utf-8");
		}
		header("Last-Modified: ".gmdate( "D, d M Y H:i:s", filemtime( "./pages" ) )." GMT" );
		header("Cache-Control: public");
	}
	
	/*
	* @access : protected
	* @return : void
	* @description :
	* 変数の読み込みと、その設定。
	*/
	function prepare(){
		$file = new SerializedFile( "./option" );
		$this->option = $file->read();
		$file = new SerializedFile( "./pages" );
		$this->pages = $file->read();
		$file = new SerializedFile( "./categories" );
		$this->categories = $file->read();
		unset($file);
		
		$this->categorizedPage = array();
		foreach($this->pages as $p){
			foreach($p->categories as $c){
				if( array_key_exists( bin2hex($c->getPath()), $this->categorizedPage ) ){
					array_push($this->categorizedPage[ bin2hex($c->getPath()) ], $p);
				} else {
					$this->categorizedPage[ bin2hex($c->getPath()) ] = array($p);
				}
			}
		}
	}
	
	/*
	* @access : protected
	* @return : void
	* @description :
	* スクリプト内で使用する GET 時の引数を無害化します。
	* page と c しか使っていませんので、他はチェックしません。
	*/
	function sanitize(){
		if( array_key_exists( "page", $_GET ) == false ){
			$_GET['page'] = 1;
		} else {
			if( ereg("^[1-9][0-9]*$", $_GET['page']) == false ){
				$_GET['page'] = 1;
			} else {
				$_GET['page'] = (int) $_GET['page'];
			}
		}
		
		if( array_key_exists( "c", $_GET ) == false ){
			$category = $this->getDefaultCategory();
			$_GET['c'] = $category->getPath();
		} else {
			$category = $this->getCategoryByPath( $_GET['c'] );
			if( $category === null ){
				httpError(404, "そのカテゴリは存在しません。");
			}
		}
	}
	
	/**
	* @return Category デフォルトのカテゴリオブジェクト
	* 
	* 第1引数は内部的に使用するもので、通常は無指定で使います。
	* 何故か必ずあるはずのデフォルトカテゴリが見つからない場合 null を返します。
	**/
	function getDefaultCategory( $categories = null ){
		if( $categories === null ){
			$categories = & $this->categories;
		}
		foreach( $categories as $c ){
			$children = $c->getChildren();
			if( empty($children) ){
				if( $c->isDefault ){
					return $c;
				}
			} else {
				$canditate = $this->getDefaultCategory( $children );
				if( $canditate !== null ){
					return $canditate;
				} 
			}
		}
		return null;
	}
	
	/**
	* @return Category パスが一致したカテゴリオブジェクト
	* 
	* name とパスが一致するカテゴリを返します。子を持つか否かは関係ありません。
	* 第2引数は内部的に使用するもので、通常は無指定で使います。
	* パスと一致するものが見つからない場合 null を返します。
	**/
	function getCategoryByPath( $path, $parentCategories = null ){
		if( $parentCategories === null ){
			$parentCategories = & $this->categories;
		}
		foreach($parentCategories as $category){
			if( $category->getPath() === $path ){
				return $category;
			}
			$children = $category->getChildren();
			if( empty($children) == false ){
				$canditate = $this->getCategoryByPath( $path, $children );
				if( $canditate !== null ){
					return $canditate;
				}
			}
		}
		return null;
	}
	
	/*
	* @access : protected
	* @return : void
	* @description :
	* イテレータを使ってくるくる回して出力するところ。
	* 出力をバッファに取って、スキンに適用して最後にすべてを出力、終了します。
	*/
	function output(){
		$output = implode( "", file($this->option['template']) );
		$binary = bin2hex( $_GET['c'] );
		if( array_key_exists( $binary , $this->categorizedPage ) == false ){
			httpError( 404, "1 ページも表示するデータがありません。");
		}
		
		$category = $this->getCategoryByPath( $_GET['c'] );
			
		$iterator = new PageIterator( count($this->categorizedPage[$binary]), $_GET['page'], $this->option['limit'] );
		
		// メインリンクリスト作成
		ob_start();
		while( $iterator->hasNextElement() ){
			$idx = $iterator->nextElement();
			println( "<li>".$this->categorizedPage[$binary][$idx]->toHTML($category)."</li>" );
		}
		$linkList = ob_get_contents();
		ob_end_clean();
		
		// ページ間リンク作成
		ob_start();
		$n = $iterator->getPageCount();
		for($i=1; $i<=$n; $i++ ){
			if( $i == $_GET['page'] ){
				println("<li>".$i."</li>");
			} else {
				println("<li><a href='".basename($_SERVER['PHP_SELF'])."?page=".$i."&amp;c=".$_GET['c']."'>".$i."</a></li>");
			}
		}
		$pageIndex = ob_get_contents();
		ob_end_clean();
		
		// カテゴリリンク作成
		ob_start();
		foreach($this->categories as $c){
			println( $c->toHTML( $_GET['c'] ) );
		}
		$cList = ob_get_contents();
		ob_end_clean();
		
		// 更新時刻作成
		$lastmod = gmdate( "D, d M Y H:i:s",filemtime( "./pages" ) )." GMT";

		// 最後に置換を行い、スキンを適用する
		$output = str_replace(
			array("@@PAGE_LIST@@", "@@PAGE_INDEX@@", "@@LAST_MODIFIED@@", "@@SCRIPT_INFO@@", "@@CATEGORY_LIST@@"),
			array( $linkList, $pageIndex, $lastmod, THUMBLINK_INFO, $cList ),
			$output
		);
		print( $output );
	}
}

if( empty($_GET['src']) ){
	$application = new Thumblink();
	$application->service();
} else {
	header("Content-Type:text/html; charser=utf-8");
	show_source( __FILE__ );
}

?>
