<?php

require_once(dirname(__FILE__)."/Const.php");
require_once(dirname(__FILE__)."/HttpFunction.php");

/* 基本ファイル処理クラス */
class File{
	var $src; // private resource [file-pointer]
	var $name; // public String
	var $lockfile; // public String :: これがない場合のロック処理はflock形式(Win環境では常に失敗する)。あればrename形式
	
	var $DEAD_LOCK_TIME = 64; // public static int (const)
	
	/* public __constructer */
	/* コンストラクタ。 */
	/*
	string $filename :: 開くファイル名
	string $openmode :: 開くモード。w+ や rb+ や a といった通常のもの
	string $lockfile :: ロックファイル名。これを指定せずに以後にロックしようとした場合はflock形式で行われる。あればrename形式。ここで指定するファイルはコンストラクタ作成時点で存在している必要があります
	boolean $execlock :: ファイルを開く直前にロックを行うか。w で開く場合などに使用する。ロックファイルが指定されている必要がある。
	*/
	function File( $filename, $openmode, $lockfile = null, $execlock = false ){
		if( empty($lockfile) == false ){
			$this->lockfile = $lockfile;
		}
		if( $execlock == true ){
			if( $this->lock() == NG ){
				httpError( 500, "{$filename} を開く際に {$lockfile} をロックできませんでした。" );
			}
		}
		if( ($this->src = @fopen( $filename, $openmode )) === false ){
			httpError( 500, "{$filename} :: ファイルオープンに失敗しました。" );
		}
		$this->name = $filename;
	}
	
	/* public void */
	/* ファイルに書き込む */
	/*
	string $str :: 書き込む文字
	*/
	function write( $str ){
		fwrite( $this->src, $str );
	}
	
	/* public void */
	/* 改行付きでファイルに書き込む */
	/*
	string $str :: 書き込む文字
	string $break :: 改行コード文字。LF / CR / CRLF のどれか。デフォルトは LF
	*/
	function writeln( $str, $break = LF ){
		$this->write( "{$str}{$break}" );
	}
	
	/* public void */
	/* ファイルを閉じる。成功すれば true, 失敗したら false */
	function close(){
		return @fclose( $this->src );
	}
	
	/* public String */
	/* ファイルポインタから一行読み込んで返す */
	/*
	int $length :: 最大読み込みバイト数（デフォルト:10240で10Kbyte丁度）
	mixed $stripTags :: boolean OR string。タグを取り除くかどうか。取り除かない例外タグの指定もするなら<form><a>という風に文字列で指定。
	*/
	function readLine( $length = 10240, $stripTags = false ){
		if( $stripTags === false ){
			return fgets( $this->src, $length );
		} else if( $stripTags === true ){
			return fgetss( $this->src, $length );
		} else {
			return fgetss( $this->src, $length, $stripTags );
		}
	}
	
	/* public array */
	/* ファイルポインタから一行読み込み、文字列で分割して返す */
	/*
	int $length :: 最大読み込みバイト数(デフォルト:10240で10Kbyte丁度)
	string $delimiter :: 区切り文字。デフォルトはカンマ1文字
	string $enclosure :: 文字列の左右を囲う文字列。デフォルトはダブルクォーテーション1文字
	*/
	function readCSV( $length = 10240, $delimiter = ",", $enclosure = "\"" ){
		return fgetcsv( $this->src, $length, $delimiter, $enclosure );
	}
	
	/* public String */
	/* ファイルポインタから指定バイト分だけ読み取って返す。主にバイナリに使用 */
	/*
	int $length :: 最大読み込みバイト数。デフォルトは1024で1Kbyte丁度
	*/
	function readBinary( $length = 1024 ){
		return fread( $this->src, $length );
	}
	
	/* public String */
	/* ファイルを現在のポインタ位置から最後まで全て読み取って返す */
	/*
	boolean $isBinary :: バイナリモードで読むならtrue, テキストモードならfalse（デフォルト）。
	*/
	function readAll( $isBinary = false ){
		$temp = "";
		$readfunc = $isBinary == true ? "readBinary" : "readLine";
		while( $this->isEOF() != false ){
			$temp .= $this->$readfunc();
		}
		return $temp;
	}
	
	/* public boolean */
	/* ファイルポインタが終端に達しているかどうかを返す */
	function isEOF(){
		return feof( $this->src );
	}
	
	/* public void */
	/* ファイルポインタの移動を行う。失敗したら強制終了 */
	function seek( $length, $basePoint = SEEK_SET ){
		if( fseek( $this->src, $length, $basePoint ) != 0 ){
			httpError(500,"ファイルポインタ位置を移動できません。(seek)");
		}
	}
	
	/* public void */
	/* ファイルポインタの先頭への移動を行う。失敗したら強制終了 */
	function rewind(){
		if( rewind( $this->src ) === false ){
			httpError(500,"ファイルポインタ位置を移動できません。(rewind)");
		}
	}
	
	/* public int */
	/* ファイルポインタの位置を返す。ポインタ位置の取得に失敗したら強制終了 */
	function tell(){
		if( ($pointer = ftell( $this->src )) === false ){
			httpError(500,"ファイルポインタ位置が取得できません。");
		}
		return $pointer;
	}
	
	/* public boolean */
	/* ファイルのロックを試行して成功すればtrue、失敗すればfalseを返す。デフォルトでは排他ロックを試みる。 */
	function lock( $mode = LOCK_EX ){
		if( empty($this->lockfile) == false ){
			return $this->nameLock();
		} else {
			return flock( $this->src, $mode );
		}
	}
	
	/* public boolean */
	/* ロックの解除を試行して成功すればtrue、失敗すればfalseを返す */
	function unlock(){
		if( empty($this->lockfile) == false ){
			return $this->unNameLock();
		} else {
			return flock( $this->src, LOCK_UN );
		}
	}
	
	/* private boolean */
	/* lock関数から呼ばれる。
	
		5回renameでのロックを試みる。
		それでもrenameが出来なければ
		存在すると思われるロックファイルの更新時刻を確かめて
		更新から時間が経っていればデッドロックとみなしてロックを解除する。
		解除した後にもう一度ロックを再帰的に試みる。
		この時ロック解除さえも出来ないようなら諦めてNG(false)を返す。
		そもそもロックファイルが見つからない場合や、
		まだロックファイル更新からあまり時間が経っていない場合もNG(false)を返す。
		
		最終的にrenameが成功すればロックファイルの更新時刻を現在にセットしてOK(true)を返す。
	*/
	function nameLock(){
		$retry = 5;
		while( @rename( $this->lockfile, "{$this->lockfile}.lock" ) == NG ){
			if( --$retry <= 0 ){
				break;
			}
			usleep(400);
		}
		if( $retry <= 0 ){
			if( $this->DEAD_LOCK_TIME <= time() - @filemtime("{$this->lockfile}.lock") ){
				if( $this->unNameLock() == OK ){
					return $this->nameLock();
				} else {
					return NG;
				}
			} else {
				return NG;
			}
		} else {
			if( file_exists( "{$this->lockfile}.lock" ) == true ){
				touch( "{$this->lockfile}.lock" );
			}
			return OK;
		}
	}
	
	/* private boolean */
	/* unlock 関数から呼ばれる。 */
	function unNameLock(){
		return @rename( "{$this->lockfile}.lock", $this->lockfile );
	}
}

?>
