<?php

require_once(dirname(__FILE__)."/Const.php");
require_once(dirname(__FILE__)."/BasicFunction.php");

/*
@access: public
@param date:long: 最終更新時刻
@param error:int: 許容する誤差（分単位）。指定しなければ 3
@description:
Last-Modified の自動対応。$date と If-Modified-Since ヘッダの差が
$error 分以上あればキャッシュ使用を要求して終了する
*/

function useCacheIfModified( $date, $error = 3 ){
	if( array_key_exists( "HTTP_IF_MODIFIED_SINCE", $_SERVER ) == true ){
		if( abs(@strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) - $date) > MINUTE * $error ){
			header("HTTP/1.1 304 Not Modified");
			exit();
		}
	}
	header("Last-Modified: ".gmdate("D, d M Y H:i:s", $date)." GMT" );
}


/* 安全なリファラかどうか。引数はURLの正規表現） */
function isSafeReferrer( $self ){
	if( array_key_exists( "HTTP_REFERER", $_SERVER ) ){
		if( eregi("^{$self}|^\$|\bBlock",$_SERVER['HTTP_REFERER']) == true ){
			return true;
		} else {
			return false;
		}
	} else {
		return true; // リファラを送らないアプリケーションが有り得るため一応trueを返す
	}
}

/* 直にファイルにアクセスされているかどうか */
/* ほとんどの場合、__FILE__を引数に渡して動作させる */
function isDirectAccess( $filename ){
	return basename($_SERVER['SCRIPT_NAME']) == basename($filename);
}

/*
引数
	$mediatype :: 対象メディアタイプ。例えば application/xhtml+xml
返り値 :: boolean
	対象のメディアタイプが Accept リクエストヘッダに含まれていなければ false, 含まれているか、Accept 自体がない場合は true
*/
function isAcceptable( $mediatype ){
	if( array_key_exists( "HTTP_ACCEPT", $_SERVER ) == true ){
		if( empty($_SERVER['HTTP_ACCEPT']) == false ){
			if( preg_match( "/".preg_quote( $mediatype, "/" )."/i" , $_SERVER['HTTP_ACCEPT'] ) == true ){
				return true;
			}
		} else {
			return false;
		}
	} else {
		return true; // コマンドライン処理時など Accept 自体がない場合はとりあえず true
	}
}

/* リクエストがそれぞれPOST,GET,HEADメソッドかどうか */
/* 単に記述が面倒なので書いている関数群。 */
function isPost(){
	return (array_key_exists( "REQUEST_METHOD", $_SERVER) and $_SERVER['REQUEST_METHOD'] == "POST");
}
function isGet(){
	return (array_key_exists( "REQUEST_METHOD", $_SERVER) and $_SERVER['REQUEST_METHOD'] == "GET");
}
function isHead(){
	return (array_key_exists( "REQUEST_METHOD", $_SERVER) and $_SERVER['REQUEST_METHOD'] == "HEAD");
}

/* ApacheもどきHTTPエラー出力&強制終了 */
function httpError( $code = 404, $userErrMsg = null ){
	/* class化して関数別に分けた方がいいかもしれない */
	switch( $code ){
		case 400:
			$error = "Bad Request";
			$msg = "リクエストは、文法が不正なためサーバに理解されませんでした。";
			break;
		case 401:	// Unauthorized - 認証されなかった
		case 402:	// Payment Required - 有料である（将来の為に予約されている）
		case 403:
		case 407:	// Proxy Authentication Required - プロキシ認証が必要
			$code = 403;
			$error = "Forbidden";
			$msg = "リクエストは拒否されました。";
			break;
		default:	// 他に適切なレスポンスがない
		case 404:
			$code = 404;
			$error = "Not Found";
			$msg = "リクエストされたリソースはサーバに存在していません。";
			break;
		case 405:
			$error = "Method Not Allowed";
			$msg = "リソースへのリクエスト方法が間違っています。";
			break;
		case 406: // application/xhtml+xml なスクリプトで IE をﾇｯｺﾛｽ時とかによく使う
			$error = "Not Acceptable";
			$msg = "アプリケーションは、サーバの提供するデータを正しく受信することが出来ません。";
			break;
		case 408:
			$error = "Request Timeout";
			$msg = "アプリケーションは、サーバの規定時間内にリクエストを発行しませんでした。";
			break;
		case 409:
			$error = "Conflict";
			$msg = "リクエストは、現在のリソースの状態と矛盾しています。何らかの競合を起こした可能性があります。";
			break;
		case 410:
			$error = "Gone";
			$msg = "リクエストされたリソースの行方はもはや不明です。";
			break;
		case 411: // 主に POST リクエスト時
			$error = "Length Required";
			$msg = "リクエストされた際に、妥当なContent-Lengthヘッダがありませんでした。";
			break;
		case 412:
			$error = "Precondition Failed";
			$msg = "リクエストされた処理は、実行する準備が出来ませんでした。";
			break;
		case 413: // アップローダー等でエラーを投げる時によく使う
			$error = "Request Entity Too Large";
			$msg = "送信しようとした情報は、容量が大きすぎます。";
			break;
		case 414:
			$error = "Request-URI Too Large";
			$msg = "リクエストされたURIが長すぎます。";
			break;
		case 415:
			$error = "Unsupported Media Type";
			$msg = "アプリケーションは、リクエストされたリソースの形式をサポートしていません。";
			break;
		case 416:
			$error = "Requested Range Not Satisfiable";
			$msg = "リクエストされたRangeヘッダフィールド類が異常な値です。";
			break;
		case 417:
			$error = "Expectation Failed";
			$msg = "リクエストされた際のExpectヘッダフィールドを正しく処理できませんでした。";
			break;
		case 418: // 有名なジョーク RFC より
			$error = "I'm a teapot";
			$msg = "さぁどうぞ、コーヒーです。";
			break;
		case 422: // POST されたデータが XML でDTDに準拠していない場合など
			$error = "Unprocessable Entity";
			$msg = "リクエストされた際のデータに妥当性がありませんでした。";
		case 423:
			$error = "Locked";
			$msg = "ロックされたリソースに対して不適当な操作を行おうとしました。";
		case 424:
			$error = "Failed Dependency";
			$msg = "リクエストされた操作の処理途中にエラーが発生したため連鎖的に一連の操作は失敗しました。";
		case 426:
			$error = "Upgrade Required";
			$msg = "リクエストされた操作には TLS プロトコルのサポートが必要です。";
		case 500:
			$error = "Internal Server Error";
			$msg = "サーバは、リクエストの処理途中でエラーを発生させました。";
			break;
		case 501:
			$error = "Not Implemented";
			$msg = "サーバは、リクエストされた操作を理解できませんでした。";
			break;
		case 502:
			$error = "Bad Gateway";
			$msg = "サーバは、より上流のサーバへリクエストを転送しましたが、不正な応答を受信しました。";
			break;
		case 503:
			$error = "Service Unavailable";
			$msg = "サーバは一時的な過負荷か、メンテナンス中であるため、現在リクエストを扱うことができません。";
			break;
		case 504:
			$error = "Gateway Timeout";
			$msg = "サーバは、より上流のサーバへリクエストを転送しましたが応答がありませんでした。";
			break;
		case 505:
			$error = "HTTP Version Not Suppurted";
			$msg = "サーバは、リクエストの際のHTTPバージョンに対応していません。";
			break;
	}
	allBufferClear();
	/* エラーを表示して強制終了。ちょっと怪しめな$_SERVER変数を使っているのでApacheでないと正常動作しないかも */
	ob_start();
	header("Status: {$code}");
	header("HTTP/1.1 {$code} {$error}");
	header("Content-Type:text/html; charset=utf-8" );
	header("Connection: close");
	if( isHead() == false ){
		println("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">");
		println("<html lang='ja'>");
		println("<head>");
		println("<meta http-equiv='Content-Type' content='text/html; charset=utf-8'>");
		println("<title>{$code} {$error}</title>");
		println("</head>");
		println("<body>");
		println("<h1>{$code} {$error}</h1>");
		println("<p>{$msg}</p>");
		if( empty($userErrMsg) == false ){
			println("<p>{$userErrMsg}</p>");
		}
		if( isPost() == true ){
			foreach($_POST as $key => $val){
				$encoding = mb_detect_encoding($val, "JIS,UTF-8,EUC-JP,SJIS");
				$_POST[$key] = mb_convert_encoding($val, "UTF-8", $encoding);
			}
			println("<p>送信しようとした情報は以下の通りです。</p>");
			println("<pre>");
			print_r($_POST);
			println("</pre>");
		}
		println("<p>{$_SERVER['PHP_SELF']}</p>");
		println("<hr>");
		if( empty($_SERVER['SERVER_SIGNATURE']) == false ){
			println("{$_SERVER['SERVER_SIGNATURE']}");
		}
		println("</body>");
		println("</html>");
	}
	$out = ob_get_contents();
	ob_end_clean();
	print( $out );
	exit( $code - 300 );
}

/* ApacheもどきHTTPリダイレクト */
function httpRedirect( $url, $code = 302, $choices = null ){
	allBufferClear();
	
	ob_start();
	if( is_int($code) == false ){
		$code = 302;
	}
	switch( $code ){
		case 300:
			$status = "Multiple Choices";
			$msg = "サーバは複数の選択肢をユーザに提供します。";
			if( empty($choices) == true or is_array($choices) == false or count($choices) <= 1 ){
				httpError(500,"300 Multiple Choices のレスポンスを返そうとしましたが、対象のURLが複数ありませんでした。");
			}
			break;
		case 301: // 因みに移動先が分からずLocationを示せない恒久的移動は410エラー。
			$status = "Permanently Moved";
			$msg = "リクエストされたリソースは恒久的な移動をしています。";
			break;
		default:
		case 302: // 示されたLocationに対し最初と同じメソッドでリクエストを行わせますが303と同様に扱うUAが多いです
			$status = "Found";
			$msg = "リクエストされたリソースは一時的な移動をしています。"; 
			break;
		case 303: // 明示的にGETメソッドを使用させてLocationにリクエストを行わせます
			$status = "See Other";
			$msg = "リクエストに対して適切なリソースは他のURIを参照することで提供されます。";
			break;
		case 304: // キャッシュを使わせます。この時レスポンスボディは含んではなりません……よって$msgは使われません
			$status = "Not Modified";
			$msg = "リクエストされた文書は以前参照された時からの更新がありません。";
			break;
		case 305:
			$status = "Use Proxy";
			$msg = "Location により与えられるプロキシを使用してアクセスしてください。";
			break;

		/* 306 ステータスコードはもはや存在せず、将来のために予約されています */

		case 307: // 示されたLocationに対し *明示的に* 最初と同じメソッドでリクエストを行わせます
			$status = "Temporary Redirect";
			$msg = "リクエストされたリソースは、一時的な移動をしています。";
			break;
	}
	header("Status: {$code}");
	header("HTTP/1.1 {$code} {$status}");
	if( $code != 300 ){
		header("Location: {$url}");
	}
	header("Content-Type: text/html; charset=utf-8");
	header("Connection: close");
	if( isHead() == false and $code != 304 ){
		println("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">");
		println("<html lang='ja'>");
		println("<head>");
		println("<meta http-equiv='Content-Type' content='text/html; charset=utf-8'>");
		if( $code != 300 ){
			println("<meta http-equiv='Refresh' content='1; url=$url'>");
		}
		println("<title>{$code} {$status}</title>");
		println("</head>");
		println("<body>");
		println("<h1>{$code} {$status}</h1>");
		if( empty($msg) == false ){
			println("<p>{$msg}</p>");
		}
		if( $code == 300 ){
			println("<p>Choice URL to complete your request.</p>");
			println("<ul>");
			foreach( $choices as $location ){
				println("<li><a href='{$url}'>{$url}</a></li>");
			}
			println("</ul>");
		} else {
			println("<p>Wait or move <a href='{$url}'>{$url}</a> .</p>");
		}
		println("<p>{$_SERVER['PHP_SELF']}</p>");
		println("<hr>");
		println("{$_SERVER['SERVER_SIGNATURE']}");
		println("</body>");
		println("</html>");
	}
	$out = mb_convert_encoding( ob_get_contents() , "UTF-8" , "EUC-JP,UTF-8" );
	ob_end_clean();
	print( $out );
	exit();
}

?>
