<?php

require_once(dirname(__FILE__)."/File.php");

/* serialize されたファイルの読み書き処理 */
class SerializedFile{
	var $file;
	var $lockfile;
	
	function SerializedFile( $filename, $lockfile = null ){
		$this->file = $filename;
		if( empty($lockfile) == false ){
			$this->lockfile = $lockfile;
		}
	}
	
	/* public mixed */
	/* serializeされたファイルを読み込んで全ての情報を返す */
	/*
	boolean $doLock :: 共有ロック処理を行うかどうか。デフォルトでは行わない。true指定の時、ロックできなかったら強制終了（成功時は終了時に自動でロックを解除します）
	string $encoding :: 読み込み先ファイルのエンコーディング
	*/
	function read( $doLock = false, $encoding = "EUC-JP" ){
		$curEncode = mb_internal_encoding();
		mb_internal_encoding($encoding);
		if( empty($this->lockfile) == true ){
			$f = new File( $this->file, "rb" );
		} else {
			$f = new File( $this->file, "rb", $this->lockfile );
		}
		if( $doLock == true ){
			if( $f->lock( LOCK_SH ) == NG ){
				httpError(503,"{$this->file} の共有ロックができません。");
			}
		}
		$s = "";
		while( $f->isEOF() == false ){
			$s .= $f->readBinary();
		}
		if( $doLock == true ){
			$f->unlock();
		}
		$f->close();
		mb_internal_encoding($curEncode);
		return unserialize($s);
	}
	
	/* public void */
	/* 指定されたオブジェクトをserializeしてファイルへ書き込む */
	/* 事前にロック処理をしないと、場合によってはデータ破損の危険あり。(wで開くから) */
	/*
	mixed & $data :: 書き込むデータ(変数である必要がある)
	*/
	function write( &$data, $doLock = false ){
		if( empty($this->lockfile) == false and $doLock == true ){
			$f = new File( $this->file, "w", $this->lockfile, true );
		} else {
			$f = new File( $this->file, "w" );
		}
		$f->write( serialize($data) );
		if( empty($this->lockfile) == false ){
			$f->unlock();
		}
		$f->close();
	}
}

?>
